#!/usr/bin/python

# SPDX-FileCopyrightText: 2020 Simone Persiani <iosonopersia@gmail.com>
# SPDX-FileCopyrightText: 2025 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

# -*- coding: utf-8 -*-
