#!/usr/bin/python

# SPDX-FileCopyrightText: 2020-2021 Simone Persiani <iosonopersia@gmail.com>
#
# SPDX-License-Identifier: ISC

# -*- coding: utf-8 -*-

from oc_ocdm.prov.entities.snapshot_entity import SnapshotEntity
