#!/usr/bin/python

# SPDX-FileCopyrightText: 2025 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

# -*- coding: utf-8 -*-
import unittest


class TestGraphEntity(unittest.TestCase):
    pass


if __name__ == '__main__':
    unittest.main()
